@@ -1,81 +1,27 @@
 # coding: utf-8
 """Compatibility library."""
-from __future__ import absolute_import
-
-import inspect
-import sys
-
-import numpy as np
-
-is_py3 = (sys.version_info[0] == 3)
-
-"""Compatibility between Python2 and Python3"""
-if is_py3:
-    zip_ = zip
-    string_type = str
-    numeric_types = (int, float, bool)
-    integer_types = (int, )
-    range_ = range
-
-    def argc_(func):
-        """Count the number of arguments of a function."""
-        return len(inspect.signature(func).parameters)
-
-    def decode_string(bytestring):
-        """Decode C bytestring to ordinary string."""
-        return bytestring.decode('utf-8')
-else:
-    from itertools import izip as zip_
-    string_type = basestring
-    numeric_types = (int, long, float, bool)
-    integer_types = (int, long)
-    range_ = xrange
-
-    def argc_(func):
-        """Count the number of arguments of a function."""
-        return len(inspect.getargspec(func).args)
-
-    def decode_string(bytestring):
-        """Decode C bytestring to ordinary string."""
-        return bytestring
-
-"""json"""
-try:
-    import simplejson as json
-except (ImportError, SyntaxError):
-    # simplejson does not support Python 3.2, it throws a SyntaxError
-    # because of u'...' Unicode literals.
-    import json
-
-
-def json_default_with_numpy(obj):
-    """Convert numpy classes to JSON serializable objects."""
-    if isinstance(obj, (np.integer, np.floating, np.bool_)):
-        return obj.item()
-    elif isinstance(obj, np.ndarray):
-        return obj.tolist()
-    else:
-        return obj
-
 
 """pandas"""
 try:
-    from pandas import Series, DataFrame
+    from pandas import DataFrame as pd_DataFrame
+    from pandas import Series as pd_Series
+    from pandas import concat
     from pandas.api.types import is_sparse as is_dtype_sparse
     PANDAS_INSTALLED = True
 except ImportError:
     PANDAS_INSTALLED = False
 
-    class Series(object):
+    class pd_Series:  # type: ignore
         """Dummy class for pandas.Series."""
 
         pass
 
-    class DataFrame(object):
+    class pd_DataFrame:  # type: ignore
         """Dummy class for pandas.DataFrame."""
 
         pass
 
+    concat = None
     is_dtype_sparse = None
 
 """matplotlib"""
@@ -96,32 +42,31 @@ except ImportError:
 try:
     import datatable
     if hasattr(datatable, "Frame"):
-        DataTable = datatable.Frame
+        dt_DataTable = datatable.Frame
     else:
-        DataTable = datatable.DataTable
+        dt_DataTable = datatable.DataTable
     DATATABLE_INSTALLED = True
 except ImportError:
     DATATABLE_INSTALLED = False
 
-    class DataTable(object):
-        """Dummy class for DataTable."""
+    class dt_DataTable:  # type: ignore
+        """Dummy class for datatable.DataTable."""
 
         pass
 
 
 """sklearn"""
 try:
-    from sklearn.base import BaseEstimator
-    from sklearn.base import RegressorMixin, ClassifierMixin
+    from sklearn.base import BaseEstimator, ClassifierMixin, RegressorMixin
     from sklearn.preprocessing import LabelEncoder
     from sklearn.utils.class_weight import compute_sample_weight
     from sklearn.utils.multiclass import check_classification_targets
-    from sklearn.utils.validation import assert_all_finite, check_X_y, check_array
+    from sklearn.utils.validation import assert_all_finite, check_array, check_X_y
     try:
-        from sklearn.model_selection import StratifiedKFold, GroupKFold
         from sklearn.exceptions import NotFittedError
+        from sklearn.model_selection import GroupKFold, StratifiedKFold
     except ImportError:
-        from sklearn.cross_validation import StratifiedKFold, GroupKFold
+        from sklearn.cross_validation import GroupKFold, StratifiedKFold
         from sklearn.utils.validation import NotFittedError
     try:
         from sklearn.utils.validation import _check_sample_weight
@@ -149,9 +94,22 @@ try:
     _LGBMComputeSampleWeight = compute_sample_weight
 except ImportError:
     SKLEARN_INSTALLED = False
-    _LGBMModelBase = object
-    _LGBMClassifierBase = object
-    _LGBMRegressorBase = object
+
+    class _LGBMModelBase:  # type: ignore
+        """Dummy class for sklearn.base.BaseEstimator."""
+
+        pass
+
+    class _LGBMClassifierBase:  # type: ignore
+        """Dummy class for sklearn.base.ClassifierMixin."""
+
+        pass
+
+    class _LGBMRegressorBase:  # type: ignore
+        """Dummy class for sklearn.base.RegressorMixin."""
+
+        pass
+
     _LGBMLabelEncoder = None
     LGBMNotFittedError = ValueError
     _LGBMStratifiedKFold = None
@@ -163,9 +121,41 @@ except ImportError:
     _LGBMCheckClassificationTargets = None
     _LGBMComputeSampleWeight = None
 
+"""dask"""
+try:
+    from dask import delayed
+    from dask.array import Array as dask_Array
+    from dask.array import from_delayed as dask_array_from_delayed
+    from dask.bag import from_delayed as dask_bag_from_delayed
+    from dask.dataframe import DataFrame as dask_DataFrame
+    from dask.dataframe import Series as dask_Series
+    from dask.distributed import Client, default_client, wait
+    DASK_INSTALLED = True
+except ImportError:
+    DASK_INSTALLED = False
 
-# DeprecationWarning is not shown by default, so let's create our own with higher level
-class LGBMDeprecationWarning(UserWarning):
-    """Custom deprecation warning."""
+    dask_array_from_delayed = None
+    dask_bag_from_delayed = None
+    delayed = None
+    default_client = None
+    wait = None
 
-    pass
+    class Client:  # type: ignore
+        """Dummy class for dask.distributed.Client."""
+
+        pass
+
+    class dask_Array:  # type: ignore
+        """Dummy class for dask.array.Array."""
+
+        pass
+
+    class dask_DataFrame:  # type: ignore
+        """Dummy class for dask.dataframe.DataFrame."""
+
+        pass
+
+    class dask_Series:  # type: ignore
+        """Dummy class for dask.dataframe.Series."""
+
+        pass
