@@ -265,7 +265,7 @@ void CUDATreeLearner::CountDenseFeatureGroups() {
     }
   }
   if (!num_dense_feature_groups_) {
-    Log::Warning("GPU acceleration is disabled because no non-trival dense features can be found");
+    Log::Warning("GPU acceleration is disabled because no non-trivial dense features can be found");
   }
 }
 
@@ -408,7 +408,7 @@ void CUDATreeLearner::copyDenseFeature() {
     // looking for dword_features_ non-sparse feature-groups
     if (!train_data_->IsMultiGroup(i)) {
       dense_feature_group_map_.push_back(i);
-      auto sizes_in_byte = train_data_->FeatureGroupSizesInByte(i);
+      auto sizes_in_byte = std::min(train_data_->FeatureGroupSizesInByte(i), static_cast<size_t>(num_data_));
       void* tmp_data = train_data_->FeatureGroupData(i);
       Log::Debug("Started copying dense features from CPU to GPU - 2");
       CUDASUCCESS_OR_FATAL(cudaMemcpyAsync(&device_features[copied_feature * num_data_], tmp_data, sizes_in_byte, cudaMemcpyHostToDevice, stream_[device_id]));
@@ -470,10 +470,10 @@ void CUDATreeLearner::InitGPU(int num_gpu) {
     Log::Fatal("bin size %d cannot run on GPU", max_num_bin_);
   }
   if (max_num_bin_ == 65) {
-    Log::Warning("Setting max_bin to 63 is sugguested for best performance");
+    Log::Warning("Setting max_bin to 63 is suggested for best performance");
   }
   if (max_num_bin_ == 17) {
-    Log::Warning("Setting max_bin to 15 is sugguested for best performance");
+    Log::Warning("Setting max_bin to 15 is suggested for best performance");
   }
 
   // get num_dense_feature_groups_
@@ -534,8 +534,8 @@ void CUDATreeLearner::InitGPU(int num_gpu) {
   copyDenseFeature();
 }
 
-Tree* CUDATreeLearner::Train(const score_t* gradients, const score_t *hessians) {
-  Tree *ret = SerialTreeLearner::Train(gradients, hessians);
+Tree* CUDATreeLearner::Train(const score_t* gradients, const score_t *hessians, bool is_first_tree) {
+  Tree *ret = SerialTreeLearner::Train(gradients, hessians, is_first_tree);
   return ret;
 }
 
